/**************************************************************************/
/*!
    @file     cmd_eeprom_write.c
    @author   K. Townsend (microBuilder.eu)

    @brief    Code to execute for cmd_eeprom_write in the cli

    @section LICENSE

    Software License Agreement (BSD License)

    Copyright (c) 2013, K. Townsend (microBuilder.eu)
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
    1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
    3. Neither the name of the copyright holders nor the
    names of its contributors may be used to endorse or promote products
    derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/**************************************************************************/
#include <stdio.h>

#include "cli.h"
#include "commands.h"       // Generic helper functions

#include "eeprom.h"

/**************************************************************************/
/*!
    Writes a single byte at the supplied EEPROM address
*/
/**************************************************************************/
void cmd_eeprom_write(uint8_t argc, char **argv)
{
  uint8_t val;
  int32_t val32;
	int error;
  int32_t addr32;

  // Try to convert supplied address to an integer
  getNumber (argv[0], &addr32);
	
  // Check for invalid values (getNumber may complain about this as well)
  if (addr32 < 0 || addr32 > CFG_EEPROM_SIZE)
  {
    printf("Address out of range\r\n");
    return;
  }

  // Try to convert supplied data to an integer
  getNumber (argv[1], &val32);

  // Check for invalid values (getNumber may complain about this as well)
  if (val32 < 0 || val32 > 0xFF)
  {
    printf("Invalid Data: 0-255 or 0x00-0xFF required.\r\n");
    return;
  }

  // Data seems to be OK
  val = (uint8_t)val32;

  // Write data at supplied address
  error = writeEEPROM((uint8_t*)addr32, (uint8_t *)&val, 1);
  if (error)
  {
    printf("Error reading EEPROM: %d\r\n", error);
  }
  else
  {
    printf("0x%02X written at 0x%04X\r\n", val, (uint16_t)addr32 & 0xFFFF);
  }
}


