#include <string.h>
#include "LPC11Uxx.h"            
#include "cdc.h"
#include "power_api.h"
#include "mw_usbd_rom_api.h"

extern uint8_t CDC_DeviceDescriptor[];
extern uint8_t CDC_StringDescriptor[];
extern uint8_t CDC_ConfigDescriptor[];

USBD_API_T* pUsbApi;

typedef struct CDC_DATA {
  USBD_HANDLE_T hUsb;
  USBD_HANDLE_T hCdc;
	volatile uint8_t config_value;
  uint8_t* rxBuf;
  uint8_t* txBuf;
  volatile uint8_t ser_pos;
  volatile uint16_t rxlen;
  volatile uint16_t txlen;
  volatile uint32_t sof_counter;
  volatile uint32_t last_in_buf;
  volatile uint16_t usbrx_pend;
} CDC_DATA_T; 

CDC_DATA_T g_vCOM;

/**********************************************************************
 ** Function prototyping 
**********************************************************************/
ErrorCode_t CDC_bulk_in_hdlr(USBD_HANDLE_T hUsb, void* data, uint32_t event); 
ErrorCode_t CDC_bulk_out_hdlr(USBD_HANDLE_T hUsb, void* data, uint32_t event); 

void USB_pin_clk_init(void)
{
  /* Enable AHB clock to the GPIO domain. */
  LPC_SYSCON->SYSAHBCLKCTRL |= (1<<6);

  /* Enable AHB clock to the USB block and USB RAM. */
  LPC_SYSCON->SYSAHBCLKCTRL |= ((0x1<<14)|(0x1<<27));

  /* Pull-down is needed, or internally, VBUS will be floating. This is to
  address the wrong status in VBUSDebouncing bit in CmdStatus register. It
  happens on the NXP Validation Board only that a wrong ESD protection chip is used. */
  LPC_IOCON->PIO0_3   &= ~0x1F; 
//  LPC_IOCON->PIO0_3   |= ((0x1<<3)|(0x01<<0));	/* Secondary function VBUS */
  LPC_IOCON->PIO0_3   |= (0x01<<0);			/* Secondary function VBUS */
  LPC_IOCON->PIO0_6   &= ~0x07;
  LPC_IOCON->PIO0_6   |= (0x01<<0);			/* Secondary function SoftConn */

  return;
}

int CDC_USB_Configured(void)
{
	return (g_vCOM.config_value != 0); /* Check if USB is configured */
}

void CDC_usb_send(CDC_DATA_T* pVcom)
{
  /* data received send it back */
	if (g_vCOM.config_value) /* Check if USB is configured */
		pVcom->txlen -= pUsbApi->hw->WriteEP(pVcom->hUsb, USB_CDC_EP_BULK_IN, pVcom->txBuf, pVcom->txlen); 
	else
		pVcom->txlen = 0;	
}

int sendchar(int ch)
{
  if (g_vCOM.txlen < USB_HS_MAX_BULK_PACKET) { 
    g_vCOM.txBuf[g_vCOM.txlen++] = (uint8_t)ch;
  } 

  if (g_vCOM.txlen == USB_HS_MAX_BULK_PACKET) {
    CDC_usb_send(&g_vCOM);
  }
  g_vCOM.last_in_buf = g_vCOM.sof_counter;
	
	return ch;
}

int getkey(void)
{
	uint8_t *pbuf = g_vCOM.rxBuf;
	uint8_t key;

	if (g_vCOM.ser_pos < g_vCOM.rxlen)
	{
		key = pbuf[g_vCOM.ser_pos++];
		/* if done check anything pending */
		if (g_vCOM.ser_pos == g_vCOM.rxlen)
		{
			/* Tx complete free the buffer */
			g_vCOM.ser_pos = 0;
			g_vCOM.rxlen = 0;
			if (g_vCOM.usbrx_pend)
			{
				g_vCOM.usbrx_pend = 0;
				CDC_bulk_out_hdlr(g_vCOM.hUsb, (void*)&g_vCOM, USB_EVT_OUT);
			}
		}
		return key;
	}
	
  return -1; /* EOF */
}

ErrorCode_t CDC_sof_event(USBD_HANDLE_T hUsb)
{
  CDC_DATA_T* pVcom = &g_vCOM;
  uint32_t diff = pVcom->sof_counter - pVcom->last_in_buf;

  pVcom->sof_counter++;

  if (pVcom->last_in_buf && (diff > 5)) {
      CDC_usb_send(pVcom);
  }
    	
  return LPC_OK;
}

ErrorCode_t USB_Configure_Event (USBD_HANDLE_T hUsb) {

  USB_CORE_CTRL_T* pCtrl = (USB_CORE_CTRL_T*)hUsb;
	g_vCOM.config_value = pCtrl->config_value;
  if (g_vCOM.config_value) {                   /* Check if USB is configured */
    g_vCOM.ser_pos = 0;                   // initialize buffer parameters
    g_vCOM.rxlen = g_vCOM.txlen = 0;
		/* enable SOF after we are connected */
		pUsbApi->hw->EnableEvent(g_vCOM.hUsb, 0, USB_EVT_SOF, 1);
  }
	else
	{
		/* disable SOF */
		pUsbApi->hw->EnableEvent(g_vCOM.hUsb, 0, USB_EVT_SOF, 0);
	}
  return LPC_OK;
}

ErrorCode_t CDC_bulk_in_hdlr(USBD_HANDLE_T hUsb, void* data, uint32_t event) 
{
//  CDC_DATA_T* pVcom = (CDC_DATA_T*) data;
//  Not needed as WriteEP() is called in CDC_usb_send() immediately.  
//  if (event == USB_EVT_IN) {
//  }
  return LPC_OK;
}

ErrorCode_t CDC_bulk_out_hdlr(USBD_HANDLE_T hUsb, void* data, uint32_t event) 
{
  CDC_DATA_T* pVcom = (CDC_DATA_T*) data;

  switch (event) {
    case USB_EVT_OUT:

      if (pVcom->rxlen == 0) {
        pVcom->rxlen = pUsbApi->hw->ReadEP(hUsb, USB_CDC_EP_BULK_OUT, pVcom->rxBuf);
      } else {
        /* indicate bridge write buffer pending in USB buf */
        pVcom->usbrx_pend = 1;
      }
      break;
    default: 
      break;
  }
  return LPC_OK;
}

void USB_IRQHandler(void)
{
  pUsbApi->hw->ISR(g_vCOM.hUsb);
}

void CDC_Init(void)
{
  USBD_API_INIT_PARAM_T usb_param;
  USBD_CDC_INIT_PARAM_T cdc_param;
  USB_CORE_DESCS_T desc;
  USBD_HANDLE_T hUsb, hCdc;
  ErrorCode_t ret = LPC_OK;
  uint32_t ep_indx;

  SystemCoreClockUpdate ();

  /* get USB API table pointer */
  pUsbApi = (USBD_API_T*)((*(ROM **)(0x1FFF1FF8))->pUSBD);

  /* enable clocks and pinmux for usb0 */
  USB_pin_clk_init();

  /* initilize call back structures */
  memset((void*)&usb_param, 0, sizeof(USBD_API_INIT_PARAM_T));
  usb_param.usb_reg_base = LPC_USB_BASE;
  usb_param.mem_base = 0x20004000;
  usb_param.mem_size = 0x800;
  usb_param.max_num_ep = 3;

  /* init CDC params */
  memset((void*)&cdc_param, 0, sizeof(USBD_CDC_INIT_PARAM_T));
  memset((void*)&g_vCOM, 0, sizeof(CDC_DATA_T));

  /* user defined functions */
  usb_param.USB_SOF_Event = CDC_sof_event;
	usb_param.USB_Configure_Event = USB_Configure_Event;

/* Initialize Descriptor pointers */
  memset((void*)&desc, 0, sizeof(USB_CORE_DESCS_T));
  desc.device_desc = (uint8_t *)&CDC_DeviceDescriptor[0];
  desc.string_desc = (uint8_t *)&CDC_StringDescriptor[0];
  desc.full_speed_desc = (uint8_t *)&CDC_ConfigDescriptor[0];
  desc.high_speed_desc = (uint8_t *)&CDC_ConfigDescriptor[0];

  /* USB Initialization */
  ret = pUsbApi->hw->Init(&hUsb, &desc, &usb_param);  
  if (ret == LPC_OK) {

    // init CDC params
    cdc_param.mem_base = usb_param.mem_base;
    cdc_param.mem_size = usb_param.mem_size;
    cdc_param.cif_intf_desc = (uint8_t *)&CDC_ConfigDescriptor[USB_CONFIGUARTION_DESC_SIZE];
    cdc_param.dif_intf_desc = (uint8_t *)&CDC_ConfigDescriptor[USB_CONFIGUARTION_DESC_SIZE + \
                               USB_INTERFACE_DESC_SIZE + 0x0013 + USB_ENDPOINT_DESC_SIZE ];

    ret = pUsbApi->cdc->init(hUsb, &cdc_param, &hCdc);

    if (ret == LPC_OK) {
      /* store USB handle */
      g_vCOM.hUsb = hUsb;
      g_vCOM.hCdc = hCdc;

      /* allocate transfer buffers */
      g_vCOM.rxBuf = (uint8_t*)(cdc_param.mem_base + (0 * USB_HS_MAX_BULK_PACKET));
      g_vCOM.txBuf = (uint8_t*)(cdc_param.mem_base + (1 * USB_HS_MAX_BULK_PACKET));
      cdc_param.mem_size -= (4 * USB_HS_MAX_BULK_PACKET);

      /* register endpoint interrupt handler */
      ep_indx = (((USB_CDC_EP_BULK_IN & 0x0F) << 1) + 1);
      ret = pUsbApi->core->RegisterEpHandler (hUsb, ep_indx, CDC_bulk_in_hdlr, &g_vCOM);
      if (ret == LPC_OK) {
        /* register endpoint interrupt handler */
        ep_indx = ((USB_CDC_EP_BULK_OUT & 0x0F) << 1);
        ret = pUsbApi->core->RegisterEpHandler (hUsb, ep_indx, CDC_bulk_out_hdlr, &g_vCOM);
        if (ret == LPC_OK) {
					/* enable IRQ */
          NVIC_EnableIRQ(USB_IRQn); //  enable USB0 interrrupts 
          /* USB Connect */
          pUsbApi->hw->Connect(hUsb, 1);
        }
      }
    }    
  }
}



