#include "LPC11Uxx.h"                        /* LPC11xx definitions */
#include <stdio.h>
#include <string.h>

#include "cdc.h"
#include "cli.h"

#define SYSTICK_DELAY		(SystemCoreClock/100)

#define ACT_LED_ON()			LPC_GPIO->SET[0] = (1<<14)
#define ACT_LED_OFF()			LPC_GPIO->CLR[0] = (1<<14)
#define ACT_LED_TOGGLE()	LPC_GPIO->NOT[0] = (1<<14)

volatile uint32_t ticks_10ms = 0;

/* SysTick interrupt happens every 10 ms */
void SysTick_Handler(void)
{
  ticks_10ms++;	// count in 10ms-steps
/*
	if (ticks_10ms & (1<<6))	// every 640ms
		ACT_LED_ON();
	else
		ACT_LED_OFF();
*/
}

unsigned int spi_read(unsigned char chip_num)
{
	uint32_t data;

	// select
	if (chip_num == 1)
		LPC_GPIO->CLR[1] = (1<<23);
	else
		LPC_GPIO->CLR[1] = (1<<19);
		
	LPC_SSP1->DR = 0x0000;
	LPC_SSP1->DR = 0x0000;
	while ((LPC_SSP1->SR & 0x15) != 0x05); // Wait until RNE
	data = LPC_SSP1->DR << 16;
	data |= LPC_SSP1->DR;

	// de-select
	if (chip_num == 1)
		LPC_GPIO->SET[1] = (1<<23);
	else
		LPC_GPIO->SET[1] = (1<<19);

	return data;
}

/* Main Program */
int main (void) {
  uint32_t spi_done = 0;
  uint32_t i, data;
	
  /* Enable AHB clock to the GPIO domain. */
  LPC_SYSCON->SYSAHBCLKCTRL |= (1<<6);

	/* set up system clocks */
  LPC_SYSCON->PDRUNCFG     &= ~(1 << 5);          /* Power-up System Osc      */
  LPC_SYSCON->SYSOSCCTRL    = 0x00;								/* NOT bypassed, 1-20 MHz   */
  for (i = 0; i < 200; i++) __NOP();

  LPC_SYSCON->SYSPLLCLKSEL  = 0x01;               /* Set Sys.Osc as PLL Input */
  LPC_SYSCON->SYSPLLCLKUEN  = 0x01;               /* Update Clock Source      */
  LPC_SYSCON->SYSPLLCLKUEN  = 0x00;               /* Toggle Update Register   */
  LPC_SYSCON->SYSPLLCLKUEN  = 0x01;
  while (!(LPC_SYSCON->SYSPLLCLKUEN & 0x01));     /* Wait Until Updated       */

  LPC_SYSCON->SYSPLLCTRL    = 0x23;								/* M=1,P=3 => 48 MHz				*/
  LPC_SYSCON->PDRUNCFG     &= ~(1 << 7);          /* Power-up SYSPLL          */
  while (!(LPC_SYSCON->SYSPLLSTAT & 0x01));	      /* Wait Until PLL Locked    */

  LPC_SYSCON->MAINCLKSEL    = 0x03;     					/* Select PLL Clock Output  */
  LPC_SYSCON->MAINCLKUEN    = 0x01;               /* Update MCLK Clock Source */
  LPC_SYSCON->MAINCLKUEN    = 0x00;               /* Toggle Update Register   */
	LPC_SYSCON->MAINCLKUEN    = 0x01;
	while (!(LPC_SYSCON->MAINCLKUEN & 0x01));       /* Wait Until Updated       */

	LPC_SYSCON->SYSAHBCLKDIV  = 0x01;               /* DIV by 1                 */

	/* System clock to the IOCON needs to be enabled or
	most of the I/O related peripherals won't work. */
	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<16);

	SystemCoreClockUpdate();
	
	SysTick_Config( SYSTICK_DELAY );

	// for ACTIVITY-LED
	LPC_GPIO->DIR[0] |= (1<<14);
	LPC_IOCON->TRST_PIO0_14 &= ~0x07;
	LPC_IOCON->TRST_PIO0_14 |=  0x01;
	// for PROG-BUTTON
	LPC_GPIO->DIR[0] &= ~(1<<1);	/* P0.1 to input */
	LPC_IOCON->PIO0_1 &= ~0x3f;
	LPC_IOCON->PIO0_1 |= ((1<<5) | (0x02<<3));

	// for SR1-CTRL
	LPC_GPIO->DIR[0] |= (1<<18);
	LPC_IOCON->PIO0_18 &= ~0x07;
	LPC_IOCON->PIO0_18 |=  0x02; // 0x02 for CT32B0_MAT0
	// for SR2-CTRL
	LPC_GPIO->DIR[0] |= (1<<19);
	LPC_IOCON->PIO0_19 &= ~0x07;
	LPC_IOCON->PIO0_19 |=  0x02; // 0x02 for CT32B0_MAT1
	// for SR3-CTRL
	LPC_GPIO->DIR[1] |= (1<<26);
	LPC_IOCON->PIO1_26 &= ~0x07;
	LPC_IOCON->PIO1_26 |=  0x01; // 0x01 for CT32B0_MAT2

	// for SR4-CTRL
	LPC_GPIO->DIR[1] |= (1<<13);
	LPC_IOCON->PIO1_13 &= ~0x07;
	LPC_IOCON->PIO1_13 |=  0x02; // 0x02 for CT16B0_MAT0
	// for SR5-CTRL
	LPC_GPIO->DIR[1] |= (1<<14);
	LPC_IOCON->PIO1_14 &= ~0x07;
	LPC_IOCON->PIO1_14 |=  0x02; // 0x02 for CT32B0_MAT1

#define TIMER_CLOCK 3000
#define GRID_FREQ 50
#define TIMER_TICKS_PER_GRID_PERIOD (TIMER_CLOCK / GRID_FREQ)
#define TIMER_TICKS_PER_PWM_CYCLE (50 * TIMER_TICKS_PER_GRID_PERIOD)

	// setup PWMs
	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<9) | (1<<7);

	// timer CT32B0 for SR1-3
	LPC_CT32B0->TCR = 0x00; //stop timer
	LPC_CT32B0->TCR = 0x02; //reset counter
	LPC_CT32B0->TCR = 0x00; //release the reset

	LPC_CT32B0->IR = 0x0001F; //clear all int
	LPC_CT32B0->PR = (SystemCoreClock / TIMER_CLOCK) - 1; // 48 MHz / 3000 Hz => res of 0.33ms to match 50 and 60 Hz grid freq (60/50 cyc/period).
	LPC_CT32B0->MR3 = TIMER_TICKS_PER_PWM_CYCLE - 1;
	LPC_CT32B0->MCR = (1<<10); //reset TIMER1_32 on MR3

	LPC_CT32B0->MR0 = TIMER_TICKS_PER_PWM_CYCLE / 2; //TIMER_TICKS_PER_PWM_CYCLE;	// never match
	LPC_CT32B0->MR1 = TIMER_TICKS_PER_PWM_CYCLE / 4; //TIMER_TICKS_PER_PWM_CYCLE;	// never match
	LPC_CT32B0->MR2 = TIMER_TICKS_PER_PWM_CYCLE / 8; //TIMER_TICKS_PER_PWM_CYCLE;	// never match

	LPC_CT32B0->EMR = 0; //clear all MR match
	LPC_CT32B0->PWMC = 0x0007;	// enable PWM on MAT0-2

  LPC_CT32B0->TCR = 0x0001; //let timer run

	// timer CT16B0 for SR4-5
	LPC_CT16B0->TCR = 0x00; //stop timer
	LPC_CT16B0->TCR = 0x02; //reset counter
	LPC_CT16B0->TCR = 0x00; //release the reset

	LPC_CT16B0->IR = 0x0001F; //clear all int
	LPC_CT16B0->PR = (SystemCoreClock / TIMER_CLOCK) - 1; // 48 MHz / 3000 Hz => res of 0.33ms to match 50 and 60 Hz grid freq (60/50 cyc/period).
	LPC_CT16B0->MR3 = TIMER_TICKS_PER_PWM_CYCLE - 1;
	LPC_CT16B0->MCR = (1<<10); //reset TIMER1_32 on MR3

	LPC_CT16B0->MR0 = TIMER_TICKS_PER_PWM_CYCLE / 2; //TIMER_TICKS_PER_PWM_CYCLE;	// never match
	LPC_CT16B0->MR1 = TIMER_TICKS_PER_PWM_CYCLE / 4; //TIMER_TICKS_PER_PWM_CYCLE;	// never match

	LPC_CT16B0->EMR = 0; //clear all MR match
	LPC_CT16B0->PWMC = 0x0003;	// enable PWM on MAT0-2

  LPC_CT16B0->TCR = 0x0001; //let timer run

	// setup SPI1/SSP1
	LPC_SYSCON->PRESETCTRL |= (1<<2); //reset de-asserted
	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<18);

	// for IC1-SSEL
	LPC_GPIO->DIR[1] |= (1<<23);
	LPC_IOCON->PIO1_23 &= ~0x07;
	LPC_IOCON->PIO1_23 |=  0x00;
	LPC_GPIO->SET[1] = (1<<23);
	// for IC2-SSEL
	LPC_GPIO->DIR[1] |= (1<<19);
	LPC_IOCON->PIO1_19 &= ~0x07;
	LPC_IOCON->PIO1_19 |=  0x00;
	LPC_GPIO->SET[1] = (1<<19);
	// for SCK
	LPC_GPIO->DIR[1] |= (1<<20);
	LPC_IOCON->PIO1_20 &= ~0x07;
	LPC_IOCON->PIO1_20 |=  0x02; // SCK
	// for MISO
	LPC_GPIO->DIR[1] &= ~(1<<21);
	LPC_IOCON->PIO1_21 &= ~0x07;
	LPC_IOCON->PIO1_21 |=  0x02; // MISO

#define MAX31855_MAX_SPEED	5000000

	LPC_SYSCON->SSP1CLKDIV = 1; //(SystemCoreClock / MAX31855_MAX_SPEED) + 1;
	
	LPC_SSP1->CPSR = 0x2; // SSPCPSR clock prescale register, master mode, minimum divisor is 0x02. Fspi=SPI0CLK/(CPSDVSR*[SCR+1])=48M/(2*(1+1))=12M
	LPC_SSP1->CR0 = 0x000f | ((SystemCoreClock / MAX31855_MAX_SPEED / (LPC_SSP1->CPSR&0xff)) << 8); // data size 16bit, CPOL=0/CPHA=0, SCR=to max of MAX31855

	LPC_SSP1->CR1 |= (1<<1); // enable

	for (i = 0; i < 16; i++)
		data = LPC_SSP1->DR;

	CDC_Init();

//	while (CDC_USB_Configured() == 0) {;}
//	while (getchar() == EOF) {;}

	CLI_Init();
	printf("Init-State\r\n");
	
	while (1)
  {
		CLI_Task();	

		if ((ticks_10ms % 100) == 0) // every 1000ms
		{
			if (spi_done == 0)
			{
				ACT_LED_ON();
				data = spi_read(1);
				printf("\r\nIC1: 0x%08x\r\n", data);
				if (data & (1<<16))
				{
					printf("Fault - ");
					if (data & (1<<0))
						printf("Open Circuit\r\n");
					else if (data & (1<<1))
						printf("Short to Vcc\r\n");
					else if (data & (1<<2))
						printf("Short to GND\r\n");
					else
						printf("Unknown\r\n");
				}
				else
				{
					printf("Temp = %4d.%02d\r\n", (int)((data >> 20) & 0x03ff), ((data >> 18) & 0x03) * 25);
				}
				printf("Internal Temp = %3d.%04d\r\n", (int)((data >> 8) & 0x007f), ((data >> 4) & 0x0f) * 625);
				
				data = spi_read(2);
				printf("IC2: 0x%08x\r\n", data);
				if (data & (1<<16))
				{
					printf("Fault - ");
					if (data & (1<<0))
						printf("Open Circuit\r\n");
					else if (data & (1<<1))
						printf("Short to Vcc\r\n");
					else if (data & (1<<2))
						printf("Short to GND\r\n");
					else
						printf("Unknown\r\n");
				}
				else
				{
					printf("Temp = %4d.%02d\r\n", (int)((data >> 20) & 0x03ff), ((data >> 18) & 0x03) * 25);
				}
				printf("Internal Temp = %3d.%04d\r\n", (int)((data >> 8) & 0x007f), ((data >> 4) & 0x0f) * 625);
				spi_done = 1;
				ACT_LED_OFF();
			}
		}
		else
		{
			spi_done = 0;
		}
	}
}
